@@ -57,14 +57,14 @@ module Graphics.UI.Gtk.ModelView.TreeStore (
   ) where
 
 import Data.Bits
-import Data.Word (Word)
+import Data.Word (Word32)
 import Data.Maybe ( fromMaybe, isJust )
 import Data.Tree
-import Control.Monad ( liftM, when )
+import Control.Monad ( when )
 import Control.Exception (assert)
 import Data.IORef
 import Graphics.UI.Gtk.ModelView.Types
-import Graphics.UI.Gtk.Types (GObjectClass(..), TreeModelClass)
+import Graphics.UI.Gtk.Types (GObjectClass(..))
 import Graphics.UI.Gtk.ModelView.CustomStore
 import Graphics.UI.Gtk.ModelView.TreeModel
 import Graphics.UI.Gtk.ModelView.TreeDrag
@@ -102,6 +102,20 @@ data Store a = Store {
 -- * The given rose tree determines the initial content and may be the empty
 --   list. Each 'Tree' in the forest corresponds to one top-level node.
 --
+-- * The TreeStore maintains the initially given Forest and aligns the 'TreePath'
+--   bits to fit in 96-bit length 'TreeIter' storage.
+--
+-- * Additionally, a cache is used to achieve higher performance if operating on
+--   recently used TreePaths.
+--
+-- * __Note:__ due to the limited amount of bits available in TreeIter storage, only
+--   limited depth forests can be used with this implementation, the result of too deep
+--   Forests is an undefined behaviour while trying to retrieve the deeply nested nodes.
+--   For example: assuming the average requiement is 8 bits per tree level (max number of
+--   children at the level is 255), then we can only use 12 levels deep trees (96/8) -
+--   any further levels in a TreePath will not be encoded in the corresponding TreeIter
+--   storage.
+--
 treeStoreNew :: Forest a -> IO (TreeStore a)
 treeStoreNew forest = treeStoreNewDND forest
                         (Just treeStoreDefaultDragSourceIface)
@@ -217,24 +231,22 @@ treeStoreDefaultDragDestIface = DragDestIface {
 -- low level bit-twiddling utility functions
 --
 
--- TODO: figure out how these things work when Word is 64 bits
-
-bitsNeeded :: Word -> Int
+bitsNeeded :: Word32 -> Int
 bitsNeeded n = bitsNeeded' 0 n
   where bitsNeeded' b 0 = b
         bitsNeeded' b n = bitsNeeded' (b+1) (n `shiftR` 1)
 
-getBitSlice :: TreeIter -> Int -> Int -> Word
+getBitSlice :: TreeIter -> Int -> Int -> Word32
 getBitSlice (TreeIter _ a b c) off count =
       getBitSliceWord a  off     count
   .|. getBitSliceWord b (off-32) count
   .|. getBitSliceWord c (off-64) count
 
-  where getBitSliceWord :: Word -> Int -> Int -> Word
+  where getBitSliceWord :: Word32 -> Int -> Int -> Word32
         getBitSliceWord word off count =
-          word `shiftR` off .&. (1 `shiftL` count - 1)
+          word `shift` (-off) .&. (1 `shiftL` count - 1)
 
-setBitSlice :: TreeIter -> Int -> Int -> Word -> TreeIter
+setBitSlice :: TreeIter -> Int -> Int -> Word32 -> TreeIter
 setBitSlice (TreeIter stamp a b c) off count value =
   assert (value < 1 `shiftL` count) $
   TreeIter stamp
@@ -242,30 +254,30 @@ setBitSlice (TreeIter stamp a b c) off count value =
            (setBitSliceWord b (off-32) count value)
            (setBitSliceWord c (off-64) count value)
 
-  where setBitSliceWord :: Word -> Int -> Int -> Word -> Word
+  where setBitSliceWord :: Word32 -> Int -> Int -> Word32 -> Word32
         setBitSliceWord word off count value =
-          let mask = (1 `shiftL` count - 1) `shiftL` off
-           in (word .&. complement mask) .|. (value `shiftL` off)
+          let mask = (1 `shiftL` count - 1) `shift` off
+           in (word .&. complement mask) .|. (value `shift` off)
 
 
-iterPrefixEqual :: TreeIter -> TreeIter -> Int -> Bool
-iterPrefixEqual (TreeIter _ a1 b1 c1) (TreeIter _ a2 b2 c2) pos
-  | pos>64 = let mask = 1 `shiftL` (pos-64) - 1 in
-             a1==a2 && b1==b2 && (c1 .&. mask) == (c2 .&. mask)
-  | pos>32 = let mask = 1 `shiftL` (pos-32) - 1 in
-             a1==a2 && (b1 .&. mask) == (b2 .&. mask)
-  | otherwise = let mask = 1 `shiftL` pos - 1 in
-                (a1 .&. mask) == (a2 .&. mask)
+--iterPrefixEqual :: TreeIter -> TreeIter -> Int -> Bool
+--iterPrefixEqual (TreeIter _ a1 b1 c1) (TreeIter _ a2 b2 c2) pos
+--  | pos>64 = let mask = 1 `shiftL` (pos-64) - 1 in
+--             a1==a2 && b1==b2 && (c1 .&. mask) == (c2 .&. mask)
+--  | pos>32 = let mask = 1 `shiftL` (pos-32) - 1 in
+--             a1==a2 && (b1 .&. mask) == (b2 .&. mask)
+--  | otherwise = let mask = 1 `shiftL` pos - 1 in
+--                (a1 .&. mask) == (a2 .&. mask)
 
 -- | The invalid tree iterator.
 --
 invalidIter :: TreeIter
 invalidIter = TreeIter 0 0 0 0
 
-showIterBits (TreeIter _ a b c) = [showBits a, showBits b, showBits c]
-
-showBits :: Bits a => a -> String
-showBits a = [ if testBit a i then '1' else '0' | i <- [0..bitSize a - 1] ]
+--showIterBits (TreeIter _ a b c) = [showBits a, showBits b, showBits c]
+--
+--showBits :: Bits a => a -> String
+--showBits a = [ if testBit a i then '1' else '0' | i <- [0..bitSize a - 1] ]
 
 -- | Calculate the maximum number of nodes on a per-level basis.
 --
@@ -294,7 +306,7 @@ fromPath = fP 0 invalidIter
   where
   fP pos ti _ [] = Just ti -- the remaining bits are zero anyway
   fP pos ti [] _ = Nothing
-  fP pos ti (d:ds) (p:ps) = let idx = fromIntegral (p+1) :: Word in
+  fP pos ti (d:ds) (p:ps) = let idx = fromIntegral (p+1) in
     if idx >= bit d then Nothing else
     fP (pos+d) (setBitSlice ti pos d idx) ds ps
 
@@ -388,7 +400,6 @@ checkSuccess depth iter cache = case advanceCache depth iter cache of
   where
   cmp (TreeIter _ a1 b1 c1) (TreeIter _ a2 b2 c2) =
       a1==a2 && b1==b2 && c2==c2
-  cache'@((cur,sibs):_) = advanceCache depth iter cache
 
 -- | Get the leaf index of this iterator.
 --
@@ -410,7 +421,7 @@ getTreeIterLeaf ds ti = gTIL 0 0 ds
 --
 iterNext :: Depth -> TreeIter -> Cache a -> (Maybe TreeIter, Cache a)
 iterNext depth iter cache = let
-    (pos,leaf,child) = getTreeIterLeaf depth iter
+    (pos,leaf,_child) = getTreeIterLeaf depth iter
     curIdx = getBitSlice iter pos leaf
     nextIdx = curIdx+1
     nextIter = setBitSlice iter pos leaf nextIdx
@@ -426,7 +437,7 @@ iterNthChild :: Depth -> Int -> TreeIter -> Cache a  ->
                 (Maybe TreeIter, Cache a)
 iterNthChild depth childIdx_ iter cache = let
     (pos,leaf,child) = getTreeIterLeaf depth iter
-    childIdx = fromIntegral childIdx_+1 :: Word
+    childIdx = fromIntegral childIdx_+1
     nextIter = setBitSlice iter (pos+leaf) child childIdx
   in
   if childIdx>=bit child then (Nothing, cache) else
@@ -446,7 +457,7 @@ iterNChildren depth iter cache = case checkSuccess depth iter cache of
 --
 iterParent :: Depth -> TreeIter -> Maybe TreeIter
 iterParent depth iter = let
-    (pos,leaf,child) = getTreeIterLeaf depth iter
+    (pos,leaf,_child) = getTreeIterLeaf depth iter
   in if pos==0 then Nothing else
      if getBitSlice iter pos leaf==0 then Nothing else
      Just (setBitSlice iter pos leaf 0)
